# Copyright 2021 AI Redefined Inc. <dev+cogment@ai-r.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import cogment.api.common_pb2 as common_api

from cogment.errors import CogmentError
from cogment.utils import logger


class ActorParameters:
    """Class wrapping the trial api parameters for an actor"""

    def __init__(self, cog_settings, class_name, **kwargs):
        # Done not to waste resources for internal use with a '_set' call
        if cog_settings is not None:
            self._cog_settings = cog_settings
            self._raw_params = common_api.ActorParams()

            if class_name is None:
                raise CogmentError(f"Required attribute 'class_name' missing")
            if type(class_name) is not str:
                raise CogmentError(f"Wrong type for 'class_name' [{type(class_name)}]")
            self._raw_params.actor_class = class_name

            # Provide an easy way for users to set parameter attributes on construction
            for name, value in kwargs.items():
                if name[0] == "_" or name not in dir(self):
                    raise CogmentError(f"Unknown attribute [{name}]")
                setattr(self, name, value)

    def _set(self, cog_settings, raw_params):
        self._cog_settings = cog_settings
        self._raw_params = raw_params

    def __str__(self):
        result = f"ActorParameters: {self._raw_params}"
        return result

    # ----------------- properties -------------------

    @property
    def config(self):
        """Config sent to actor on trial start"""
        if self._raw_params.HasField("config"):
            config_instance = self._actor_class.config_type()
            config_instance.ParseFromString(self._raw_params.config.content)
        else:
            config_instance = None

        return config_instance

    @config.setter
    def config(self, val):
        if val is None:
            self._raw_params.ClearField("config")
        elif type(val) is self._actor_class.config_type:
            self._raw_params.config.content = val.SerializeToString()
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    @property
    def class_name(self):
        """Name of the actor class"""
        return self._raw_params.actor_class

    @class_name.setter
    def class_name(self, val):
        # We could make a setter, but we would have to sync with the config and default_action: not worth it.
        raise CogmentError(f"Cannot change class name of existing instance of ActorParameters")

    @property
    def name(self):
        """Name of the actor"""
        return self._raw_params.name

    @name.setter
    def name(self, val):
        if val is None:
            self._raw_params.ClearField("name")
        elif type(val) is str:
            self._raw_params.name = val
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    @property
    def endpoint(self):
        """Endpoint for the actor service"""
        return self._raw_params.endpoint

    @endpoint.setter
    def endpoint(self, val):
        if val is None:
            self._raw_params.ClearField("endpoint")
        elif type(val) is str:
            self._raw_params.endpoint = val
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    @property
    def implementation(self):
        """Implementation name of the actor service"""
        return self._raw_params.implementation

    @implementation.setter
    def implementation(self, val):
        if val is None:
            self._raw_params.ClearField("implementation")
        elif type(val) is str:
            self._raw_params.implementation = val
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    @property
    def initial_connection_timeout(self):
        """Timeout for connecting to a new trial"""
        return self._raw_params.initial_connection_timeout

    @initial_connection_timeout.setter
    def initial_connection_timeout(self, val):
        if val is None:
            self._raw_params.ClearField("initial_connection_timeout")
        else:
            self._raw_params.initial_connection_timeout = float(val)

    @property
    def response_timeout(self):
        """Timeout for response to observation"""
        return self._raw_params.response_timeout

    @response_timeout.setter
    def response_timeout(self, val):
        if val is None:
            self._raw_params.ClearField("response_timeout")
        else:
            self._raw_params.response_timeout = float(val)

    @property
    def optional(self):
        """If the actor is optional for the trial"""
        return self._raw_params.optional

    @optional.setter
    def optional(self, val):
        if val is None:
            self._raw_params.ClearField("optional")
        elif type(val) is bool:
            self._raw_params.optional = val
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    @property
    def default_action(self):
        """The default action space for optional actors not connected"""
        if self._raw_params.HasField("default_action"):
            action_space = self._actor_class.action_space()
            action_space.ParseFromString(self._raw_params.default_action.content)
        else:
            action_space = None

        return action_space

    @default_action.setter
    def default_action(self, val):
        if val is None:
            self._raw_params.ClearField("default_action")
        elif type(val) is self._actor_class.action_space:
            self._raw_params.default_action.content = val.SerializeToString()
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    @property
    def _actor_class(self):
        return getattr(self._cog_settings.actor_classes, self._raw_params.actor_class)


class _ActorsList:
    """List like object to manage actor parameters"""

    def __init__(self, cog_settings, raw_params):
        self._cog_settings = cog_settings
        self._raw_params = raw_params

    def __str__(self):
        result = f"Actors Parameters: {self._raw_params.actors}"
        return result

    def __len__(self):
        return len(self._raw_params.actors)

    def __getitem__(self, key):
        if type(key) is int:
            actor_params = ActorParameters(None, None)
            actor_params._set(self._cog_settings, self._raw_params.actors[key])
        elif type(key) is slice:
            actor_params = []
            actors = self._raw_params.actors
            for actual_index in range(len(actors))[key]:
                params = ActorParameters(None, None)
                params._set(self._cog_settings, actors[actual_index])
                actor_params.append(params)
        else:
            raise CogmentError(f"Wrong key type [{type(key)}]")

        return actor_params

    def __setitem__(self, key, val):
        if type(key) is int and type(val) is ActorParameters:
            self._raw_params.actors[key].CopyFrom(val._raw_params)
        elif type(key) is slice:
            # Complicated for very little gain
            raise CogmentError(f"Slices are not valid for setting")
        else:
            raise CogmentError(f"Wrong type [{type(key)}] [{type(val)}]")

    def __delitem__(self, key):
        del self._raw_params.actors[key]

    def __iter__(self):
        for raw_actor in self._raw_params.actors:
            actor_params = ActorParameters(None, None)
            actor_params._set(self._cog_settings, raw_actor)
            yield actor_params

    def append(self, val):
        if type(val) is ActorParameters:
            self._raw_params.actors.append(val._raw_params)
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    def extend(self, val):
        raw_list = []
        for actor in val:
            if type(actor) is not ActorParameters:
                raise CogmentError(f"Wrong type [{type(actor)}]")
            raw_list.append(actor._raw_params)

        self._raw_params.actors.extend(raw_list)

    def clear(self):
        self._raw_params.ClearField("actors")


class TrialParameters:
    """Class wrapping the api parameters of a trial"""

    _SERIALIZATION_TYPE = 2
    _METHODS = ["get_serialization_type", "serialize", "deserialize"]

    def __init__(self, cog_settings, **kwargs):
        # Done not to waste resources for internal use with a '_set' call
        if cog_settings is not None:
            self._cog_settings = cog_settings
            self._raw_params = common_api.TrialParams()
            self._actors = _ActorsList(cog_settings, self._raw_params)

            # Provide an easy way for users to set parameter attributes on construction
            for name, value in kwargs.items():
                # TODO: We could also protect the methods
                if name[0] == "_" or name in self._METHODS or name not in dir(self):
                    raise CogmentError(f"Unknown attribute [{name}]")
                setattr(self, name, value)

    def _set(self, cog_settings, raw_params):
        self._cog_settings = cog_settings
        self._raw_params = raw_params
        self._actors = _ActorsList(cog_settings, raw_params)

    def __str__(self):
        result = f"TrialParameters: {self._raw_params}"
        return result

    # Type of serialized data being produced and consumed by this class.
    # This is dependent on all the underlying protobuf messages used to
    # serialize/deserialize, and should be incremented if any of them changes in
    # a backward or forward incompatible way.
    # Current dependencies: TrialParams, DatalogParams, EnvironmentParams, ActorParams,
    #                       SerializedMessage
    def get_serialization_type(self):
        return TrialParameters._SERIALIZATION_TYPE

    def serialize(self):
        return self._raw_params.SerializeToString()

    def deserialize(self, raw_string, type=None):
        if type is not None and type != TrialParameters._SERIALIZATION_TYPE:
            raise CogmentError(f"Unknown serialization type")

        params = common_api.TrialParams()
        params.ParseFromString(raw_string)

        self._raw_params = params
        self._actors = _ActorsList(self._cog_settings, params)

    # ----------------- properties -------------------

    # Trial
    @property
    def config(self):
        """Config for initial trial setup by pre-trial hooks"""
        if self._raw_params.HasField("trial_config"):
            config_instance = self._cog_settings.trial.config_type()
            config_instance.ParseFromString(self._raw_params.trial_config.content)
        else:
            config_instance = None

        return config_instance

    @config.setter
    def config(self, val):
        if val is None:
            self._raw_params.ClearField("trial_config")

        elif isinstance(val, self._cog_settings.trial.config_type):
            self._raw_params.trial_config.content = val.SerializeToString()

        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    @property
    def max_steps(self):
        """Maximum number of steps before stopping the trial"""
        return self._raw_params.max_steps

    @max_steps.setter
    def max_steps(self, val):
        if val is None:
            self._raw_params.ClearField("max_steps")
        elif type(val) is int:
            self._raw_params.max_steps = val
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    @property
    def max_inactivity(self):
        """Maximum amount of time (in seconds) of inactivity before a trial is terminated"""
        return self._raw_params.max_inactivity

    @max_inactivity.setter
    def max_inactivity(self, val):
        if val is None:
            self._raw_params.ClearField("max_inactivity")
        elif type(val) is int:
            self._raw_params.max_inactivity = val
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    @property
    def nb_buffered_ticks(self):
        """Nb of buffered ticks before samples are sent to the datalog"""
        return self._raw_params.nb_buffered_ticks

    @nb_buffered_ticks.setter
    def nb_buffered_ticks(self, val):
        if val is None:
            self._raw_params.ClearField("nb_buffered_ticks")
        elif type(val) is int:
            self._raw_params.nb_buffered_ticks = val
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    # Datalog
    @property
    def datalog_endpoint(self):
        """Endpoint for the datalog service"""
        return self._raw_params.datalog.endpoint

    @datalog_endpoint.setter
    def datalog_endpoint(self, val):
        if val is None:
            self._raw_params.datalog.ClearField("endpoint")
        elif type(val) is str:
            self._raw_params.datalog.endpoint = val
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    @property
    def datalog_exclude_fields(self):
        """Name of fields to exclude from the data sent to the datalog"""
        return tuple(field for field in self._raw_params.datalog.exclude_fields)

    @datalog_exclude_fields.setter
    def datalog_exclude_fields(self, val):
        if val is None:
            self._raw_params.datalog.ClearField("exclude_fields")
        else:
            raw_fields = []
            for field in val:
                if type(field) is not str:
                    raise CogmentError(f"Wrong type for field name [{type(field)}]")
                raw_fields.append(field)

            self._raw_params.datalog.ClearField("exclude_fields")
            self._raw_params.datalog.exclude_fields.extend(raw_fields)

    # Environment
    @property
    def environment_config(self):
        """Config sent to environment on trial start"""
        if(self._raw_params.environment.HasField("config")):
            config_instance = self._cog_settings.environment.config_type()
            config_instance.ParseFromString(self._raw_params.environment.config.content)
        else:
            config_instance = None

        return config_instance

    @environment_config.setter
    def environment_config(self, val):
        if val is None:
            self._raw_params.environment.ClearField("config")

        elif isinstance(val, self._cog_settings.environment.config_type):
            self._raw_params.environment.config.content = val.SerializeToString()

        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    @property
    def environment_name(self):
        """Name of the environment"""
        return self._raw_params.environment.name

    @environment_name.setter
    def environment_name(self, val):
        if val is None:
            self._raw_params.environment.ClearField("name")
        elif type(val) is str:
            self._raw_params.environment.name = val
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    @property
    def environment_endpoint(self):
        """Endpoint for the environment service"""
        return self._raw_params.environment.endpoint

    @environment_endpoint.setter
    def environment_endpoint(self, val):
        if val is None:
            self._raw_params.environment.ClearField("endpoint")
        elif type(val) is str:
            self._raw_params.environment.endpoint = val
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    @property
    def environment_implementation(self):
        """Implementation name of the environment service"""
        return self._raw_params.environment.implementation

    @environment_implementation.setter
    def environment_implementation(self, val):
        if val is None:
            self._raw_params.environment.ClearField("implementation")
        elif type(val) is str:
            self._raw_params.environment.implementation = val
        else:
            raise CogmentError(f"Wrong type [{type(val)}]")

    # Actors
    @property
    def actors(self):
        """Parameters for all actors of the trial"""
        return self._actors

    @actors.setter
    def actors(self, val):
        for param in val:
            if type(param) is not ActorParameters:
                raise CogmentError(f"Wrong type for actor parameter [{type(param)}]")

        self._actors.clear()
        self._actors.extend(val)



# Copyright 2021 AI Redefined Inc. <dev+cogment@ai-r.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import time
import math
from typing import Any, Callable, Dict

from collections import OrderedDict
import cogment.api.model_registry_pb2 as model_registry_api
from prometheus_client import Summary

from cogment.errors import CogmentError
from cogment.utils import logger


MODEL_REGISTRY_STORE_VERSION_TIME = Summary(
    "model_registry_publish_version_seconds",
    "Time spent serializing and sending the model to the registry",
    ["model_id"],
)
MODEL_REGISTRY_RETRIEVE_VERSION_TIME = Summary(
    "model_registry_retrieve_version_seconds",
    "Time spent retrieving and deserializing the agent model version from the registry",
    ["model_id", "cached"],
)

GRPC_BYTE_SIZE_LIMIT = 4 * 1024 * 1024


class _LRU(OrderedDict):
    "Limit size, evicting the least recently looked-up key when full"

    def __init__(self, maxsize=128):
        self.maxsize = maxsize
        super().__init__()

    def __getitem__(self, key):
        value = super().__getitem__(key)
        self.move_to_end(key)
        return value

    def __setitem__(self, key, value):
        if key in self:
            self.move_to_end(key)
        super().__setitem__(key, value)
        if len(self) > self.maxsize:
            self.pop(0)


class ModelInfo:
    def __init__(self, model_id: str, model_user_data: Dict[str, str]):
        self.id = model_id
        self.user_data = model_user_data


class VersionInfo:
    def __init__(self, proto_version_info):
        self.version_number = proto_version_info.version_number
        self.creation_timestamp = proto_version_info.creation_timestamp
        self.archived = proto_version_info.archived
        self.data_hash = proto_version_info.data_hash
        self.data_size = proto_version_info.data_size


class Model(ModelInfo):
    def __init__(self, model_id: str, serialized_model: bytes, **kwargs):
        super().__init__(model_id, {})
        self.version_user_data: Dict[str, str] = {}
        self.deserialized_model = None
        self.serialized_model = serialized_model
        self.stored_version_info = None

        # Provide an easy way for users to set parameter attributes on construction
        for name, value in kwargs.items():
            if name[0] == "_" or name not in dir(self):
                raise CogmentError(f"Unknown attribute [{name}]")
            setattr(self, name, value)


class ModelRegistry:
    def __init__(self, stub):
        self._model_registry_stub = stub
        self._info_cache: Dict[str, ModelInfo] = _LRU()
        self._data_cache: Dict[str, bytes] = _LRU()

    async def store_initial_version(self, model: Model) -> VersionInfo:
        """
        Create a new model in the model registry and store the initial version

        Parameters:
            model (Model): The model
        Returns:
            version_info (VersionInfo):
            The information of the stored initial version
        """
        model_user_data_str = {}
        for key, value in model.user_data.items():
            model_user_data_str[key] = str(value)

        registry_model_info = model_registry_api.ModelInfo(model_id=model.id, user_data=model_user_data_str)
        cached_model_info = ModelInfo(model.id, model.user_data)

        req = model_registry_api.CreateOrUpdateModelRequest(model_info=registry_model_info)
        await self._model_registry_stub.CreateOrUpdateModel(req)

        self._info_cache[model.id] = cached_model_info

        version_info = await self.store_version(model)

        return version_info

    async def retrieve_model_info(self, model_id: str) -> ModelInfo:
        """
        Retrieve the given's model information

        Parameters:
            model_id (string): The model id
        Returns
            model_info (ModelInfo): The information of the model
        """
        if model_id not in self._info_cache:
            req = model_registry_api.RetrieveModelsRequest(model_ids=[model_id])
            try:
                rep = await self._model_registry_stub.RetrieveModels(req)
            except Exception:
                logger.error(f"Error retrieving model version with id [{model_id}]")
                return None

            registry_model_info = rep.model_infos[0]
            cached_model_info = ModelInfo(registry_model_info.model_id, registry_model_info.user_data)

            self._info_cache[model_id] = cached_model_info

        return self._info_cache[model_id]

    async def store_version(self, model: Model, archived=False) -> VersionInfo:
        """
        Store a new version of the model

        Parameters:
            model (Model): The model
            archive (bool - default is False):
            If true, the model version will be archived (i.e. stored in permanent storage)
        Returns
            version_info (VersionInfo): The information of the stored version
        """

        def generate_chunks():
            try:
                version_data = model.serialized_model
                version_info = model_registry_api.ModelVersionInfo(
                    model_id=model.id, archived=archived, data_size=len(version_data))
                for key, value in model.version_user_data.items():
                    version_info.user_data[key] = str(value)

                chunk_header = model_registry_api.CreateVersionRequestChunk.Header(version_info=version_info)
                yield model_registry_api.CreateVersionRequestChunk(header=chunk_header)

                chunksize = math.trunc(GRPC_BYTE_SIZE_LIMIT / 2)

                chunked_version_data = [
                    version_data[index:index + chunksize] for index in range(0, len(version_data), chunksize)
                ]
                for data_chunk in chunked_version_data:
                    chunk_body = model_registry_api.CreateVersionRequestChunk.Body(data_chunk=data_chunk)
                    yield model_registry_api.CreateVersionRequestChunk(body=chunk_body)

            except Exception as error:
                raise CogmentError(f"Failure while generating model version chunk [{error}]")

        if model.serialized_model is None:
            raise CogmentError("Cannot store a model without a serialized_model attribute")

        with MODEL_REGISTRY_STORE_VERSION_TIME.labels(model_id=model.id).time():
            rep = await self._model_registry_stub.CreateVersion(generate_chunks())

        if model.deserialized_model:
            self._data_cache[rep.version_info.data_hash] = model.deserialized_model

        return VersionInfo(rep.version_info)

    async def retrieve_version(
        self, model_id: str, version_number=-1, deserialize_func: Callable[[bytes], Any] = None,
    ) -> Model:
        """
        Retrieve a version of the model

        Parameters:
            model_id (string): Unique id of the model
            version_number (int - default is -1): The version number (-1 for the latest)
            deserialize_func (Callable[[bytes, Any]]): Function that returns an instance of the original model type
        Returns
            model (Model): The stored model
        """
        start_time = time.time()

        # First retrieve the model info and model version info
        async def retrieve_version_info(model_id, version_number):
            req = model_registry_api.RetrieveVersionInfosRequest(model_id=model_id, version_numbers=[version_number])
            try:
                rep = await self._model_registry_stub.RetrieveVersionInfos(req)
            except Exception:
                logger.error(
                    f"Failed to retrieve model version with id [{model_id}] and version number [{version_number}]"
                )
                return None

            version_info_pb = rep.version_infos[0]
            return version_info_pb

        [model_info, version_info] = await asyncio.gather(
            self.retrieve_model_info(model_id), retrieve_version_info(model_id, version_number)
        )

        if model_info is None or version_info is None:
            return None

        cached = version_info.data_hash in self._data_cache

        if cached:
            model = Model(
                model_id=model_id,
                serialized_model=None,
                stored_version_info=VersionInfo(version_info),
                user_data=model_info.user_data,
                version_user_data=version_info.user_data,
                deserialized_model=self._data_cache[version_info.data_hash]
            )

        else:
            req = model_registry_api.RetrieveVersionDataRequest(
                model_id=model_id, version_number=version_info.version_number)
            data = b""
            async for chunk in self._model_registry_stub.RetrieveVersionData(req):
                data += chunk.data_chunk

            model = Model(
                model_id=model_id,
                serialized_model=data,
                stored_version_info=VersionInfo(version_info),
                version_user_data=version_info.user_data,
                user_data=model_info.user_data,
            )

            if deserialize_func:
                deserialized_model = deserialize_func(data)
                model.deserialized_model = deserialized_model
                self._data_cache[version_info.data_hash] = deserialized_model

        MODEL_REGISTRY_RETRIEVE_VERSION_TIME.labels(model_id=model_id, cached=cached).observe(time.time() - start_time)

        return model



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/trial_datastore.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!cogment/api/trial_datastore.proto\x12\ncogmentAPI\x1a\x18\x63ogment/api/common.proto\"g\n\x15RetrieveTrialsRequest\x12\x11\n\ttrial_ids\x18\x01 \x03(\t\x12\x0f\n\x07timeout\x18\x02 \x01(\r\x12\x14\n\x0ctrials_count\x18\x03 \x01(\r\x12\x14\n\x0ctrial_handle\x18\x04 \x01(\t\"b\n\x13RetrieveTrialsReply\x12\x30\n\x0btrial_infos\x18\x01 \x03(\x0b\x32\x1b.cogmentAPI.StoredTrialInfo\x12\x19\n\x11next_trial_handle\x18\x02 \x01(\t\"\xba\x01\n\x16RetrieveSamplesRequest\x12\x11\n\ttrial_ids\x18\x01 \x03(\t\x12\x13\n\x0b\x61\x63tor_names\x18\x02 \x03(\t\x12\x15\n\ractor_classes\x18\x03 \x03(\t\x12\x1d\n\x15\x61\x63tor_implementations\x18\x04 \x03(\t\x12\x42\n\x16selected_sample_fields\x18\x05 \x03(\x0e\x32\".cogmentAPI.StoredTrialSampleField\"J\n\x13RetrieveSampleReply\x12\x33\n\x0ctrial_sample\x18\x01 \x01(\x0b\x32\x1d.cogmentAPI.StoredTrialSample\"(\n\x13\x44\x65leteTrialsRequest\x12\x11\n\ttrial_ids\x18\x01 \x03(\t\"\x13\n\x11\x44\x65leteTrialsReply\"Q\n\x0f\x41\x64\x64TrialRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12-\n\x0ctrial_params\x18\x02 \x01(\x0b\x32\x17.cogmentAPI.TrialParams\"\x0f\n\rAddTrialReply\"G\n\x10\x41\x64\x64SampleRequest\x12\x33\n\x0ctrial_sample\x18\x01 \x01(\x0b\x32\x1d.cogmentAPI.StoredTrialSample\"\x11\n\x0f\x41\x64\x64SamplesReply\"\xa0\x01\n\x0fStoredTrialInfo\x12\x10\n\x08trial_id\x18\x01 \x01(\t\x12*\n\nlast_state\x18\x02 \x01(\x0e\x32\x16.cogmentAPI.TrialState\x12\x0f\n\x07user_id\x18\x03 \x01(\t\x12\x15\n\rsamples_count\x18\x04 \x01(\r\x12\'\n\x06params\x18\x05 \x01(\x0b\x32\x17.cogmentAPI.TrialParams\"\xce\x01\n\x11StoredTrialSample\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x10\n\x08trial_id\x18\x02 \x01(\t\x12\x0f\n\x07tick_id\x18\x03 \x01(\x04\x12\x11\n\ttimestamp\x18\x04 \x01(\x06\x12%\n\x05state\x18\x05 \x01(\x0e\x32\x16.cogmentAPI.TrialState\x12\x39\n\ractor_samples\x18\x06 \x03(\x0b\x32\".cogmentAPI.StoredTrialActorSample\x12\x10\n\x08payloads\x18\x07 \x03(\x0c\"\x9d\x03\n\x16StoredTrialActorSample\x12\r\n\x05\x61\x63tor\x18\x01 \x01(\r\x12\x18\n\x0bobservation\x18\x02 \x01(\rH\x00\x88\x01\x01\x12\x13\n\x06\x61\x63tion\x18\x03 \x01(\rH\x01\x88\x01\x01\x12\x13\n\x06reward\x18\x04 \x01(\x02H\x02\x88\x01\x01\x12\x42\n\x10received_rewards\x18\x06 \x03(\x0b\x32(.cogmentAPI.StoredTrialActorSampleReward\x12>\n\x0csent_rewards\x18\x07 \x03(\x0b\x32(.cogmentAPI.StoredTrialActorSampleReward\x12\x44\n\x11received_messages\x18\x08 \x03(\x0b\x32).cogmentAPI.StoredTrialActorSampleMessage\x12@\n\rsent_messages\x18\t \x03(\x0b\x32).cogmentAPI.StoredTrialActorSampleMessageB\x0e\n\x0c_observationB\t\n\x07_actionB\t\n\x07_reward\"\x8a\x01\n\x1cStoredTrialActorSampleReward\x12\x0e\n\x06sender\x18\x01 \x01(\x05\x12\x10\n\x08receiver\x18\x02 \x01(\x05\x12\x0e\n\x06reward\x18\x04 \x01(\x02\x12\x12\n\nconfidence\x18\x05 \x01(\x02\x12\x16\n\tuser_data\x18\x06 \x01(\rH\x00\x88\x01\x01\x42\x0c\n\n_user_data\"R\n\x1dStoredTrialActorSampleMessage\x12\x0e\n\x06sender\x18\x01 \x01(\x05\x12\x10\n\x08receiver\x18\x02 \x01(\x05\x12\x0f\n\x07payload\x18\x03 \x01(\r\"\xf9\x01\n\x16TrialSamplesFileHeader\x12-\n\x0cversion_info\x18\x01 \x01(\x0b\x32\x17.cogmentAPI.VersionInfo\x12\x18\n\x10\x65xport_timestamp\x18\x02 \x01(\x06\x12I\n\x0ctrial_params\x18\x03 \x03(\x0b\x32\x33.cogmentAPI.TrialSamplesFileHeader.TrialParamsEntry\x1aK\n\x10TrialParamsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.cogmentAPI.TrialParams:\x02\x38\x01*\xf0\x02\n\x16StoredTrialSampleField\x12%\n!STORED_TRIAL_SAMPLE_FIELD_UNKNOWN\x10\x00\x12)\n%STORED_TRIAL_SAMPLE_FIELD_OBSERVATION\x10\x01\x12$\n STORED_TRIAL_SAMPLE_FIELD_ACTION\x10\x02\x12$\n STORED_TRIAL_SAMPLE_FIELD_REWARD\x10\x03\x12.\n*STORED_TRIAL_SAMPLE_FIELD_RECEIVED_REWARDS\x10\x04\x12*\n&STORED_TRIAL_SAMPLE_FIELD_SENT_REWARDS\x10\x05\x12/\n+STORED_TRIAL_SAMPLE_FIELD_RECEIVED_MESSAGES\x10\x06\x12+\n\'STORED_TRIAL_SAMPLE_FIELD_SENT_MESSAGES\x10\x07\x32\xec\x03\n\x10TrialDatastoreSP\x12V\n\x0eRetrieveTrials\x12!.cogmentAPI.RetrieveTrialsRequest\x1a\x1f.cogmentAPI.RetrieveTrialsReply\"\x00\x12Z\n\x0fRetrieveSamples\x12\".cogmentAPI.RetrieveSamplesRequest\x1a\x1f.cogmentAPI.RetrieveSampleReply\"\x00\x30\x01\x12\x44\n\x08\x41\x64\x64Trial\x12\x1b.cogmentAPI.AddTrialRequest\x1a\x19.cogmentAPI.AddTrialReply\"\x00\x12J\n\tAddSample\x12\x1c.cogmentAPI.AddSampleRequest\x1a\x1b.cogmentAPI.AddSamplesReply\"\x00(\x01\x12P\n\x0c\x44\x65leteTrials\x12\x1f.cogmentAPI.DeleteTrialsRequest\x1a\x1d.cogmentAPI.DeleteTrialsReply\"\x00\x12@\n\x07Version\x12\x1a.cogmentAPI.VersionRequest\x1a\x17.cogmentAPI.VersionInfo\"\x00\x42\x30Z.github.com/cogment/cogment/grpcapi/cogment/apib\x06proto3')

_STOREDTRIALSAMPLEFIELD = DESCRIPTOR.enum_types_by_name['StoredTrialSampleField']
StoredTrialSampleField = enum_type_wrapper.EnumTypeWrapper(_STOREDTRIALSAMPLEFIELD)
STORED_TRIAL_SAMPLE_FIELD_UNKNOWN = 0
STORED_TRIAL_SAMPLE_FIELD_OBSERVATION = 1
STORED_TRIAL_SAMPLE_FIELD_ACTION = 2
STORED_TRIAL_SAMPLE_FIELD_REWARD = 3
STORED_TRIAL_SAMPLE_FIELD_RECEIVED_REWARDS = 4
STORED_TRIAL_SAMPLE_FIELD_SENT_REWARDS = 5
STORED_TRIAL_SAMPLE_FIELD_RECEIVED_MESSAGES = 6
STORED_TRIAL_SAMPLE_FIELD_SENT_MESSAGES = 7


_RETRIEVETRIALSREQUEST = DESCRIPTOR.message_types_by_name['RetrieveTrialsRequest']
_RETRIEVETRIALSREPLY = DESCRIPTOR.message_types_by_name['RetrieveTrialsReply']
_RETRIEVESAMPLESREQUEST = DESCRIPTOR.message_types_by_name['RetrieveSamplesRequest']
_RETRIEVESAMPLEREPLY = DESCRIPTOR.message_types_by_name['RetrieveSampleReply']
_DELETETRIALSREQUEST = DESCRIPTOR.message_types_by_name['DeleteTrialsRequest']
_DELETETRIALSREPLY = DESCRIPTOR.message_types_by_name['DeleteTrialsReply']
_ADDTRIALREQUEST = DESCRIPTOR.message_types_by_name['AddTrialRequest']
_ADDTRIALREPLY = DESCRIPTOR.message_types_by_name['AddTrialReply']
_ADDSAMPLEREQUEST = DESCRIPTOR.message_types_by_name['AddSampleRequest']
_ADDSAMPLESREPLY = DESCRIPTOR.message_types_by_name['AddSamplesReply']
_STOREDTRIALINFO = DESCRIPTOR.message_types_by_name['StoredTrialInfo']
_STOREDTRIALSAMPLE = DESCRIPTOR.message_types_by_name['StoredTrialSample']
_STOREDTRIALACTORSAMPLE = DESCRIPTOR.message_types_by_name['StoredTrialActorSample']
_STOREDTRIALACTORSAMPLEREWARD = DESCRIPTOR.message_types_by_name['StoredTrialActorSampleReward']
_STOREDTRIALACTORSAMPLEMESSAGE = DESCRIPTOR.message_types_by_name['StoredTrialActorSampleMessage']
_TRIALSAMPLESFILEHEADER = DESCRIPTOR.message_types_by_name['TrialSamplesFileHeader']
_TRIALSAMPLESFILEHEADER_TRIALPARAMSENTRY = _TRIALSAMPLESFILEHEADER.nested_types_by_name['TrialParamsEntry']
RetrieveTrialsRequest = _reflection.GeneratedProtocolMessageType('RetrieveTrialsRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVETRIALSREQUEST,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveTrialsRequest)
  })
_sym_db.RegisterMessage(RetrieveTrialsRequest)

RetrieveTrialsReply = _reflection.GeneratedProtocolMessageType('RetrieveTrialsReply', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVETRIALSREPLY,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveTrialsReply)
  })
_sym_db.RegisterMessage(RetrieveTrialsReply)

RetrieveSamplesRequest = _reflection.GeneratedProtocolMessageType('RetrieveSamplesRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVESAMPLESREQUEST,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveSamplesRequest)
  })
_sym_db.RegisterMessage(RetrieveSamplesRequest)

RetrieveSampleReply = _reflection.GeneratedProtocolMessageType('RetrieveSampleReply', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVESAMPLEREPLY,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveSampleReply)
  })
_sym_db.RegisterMessage(RetrieveSampleReply)

DeleteTrialsRequest = _reflection.GeneratedProtocolMessageType('DeleteTrialsRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETETRIALSREQUEST,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.DeleteTrialsRequest)
  })
_sym_db.RegisterMessage(DeleteTrialsRequest)

DeleteTrialsReply = _reflection.GeneratedProtocolMessageType('DeleteTrialsReply', (_message.Message,), {
  'DESCRIPTOR' : _DELETETRIALSREPLY,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.DeleteTrialsReply)
  })
_sym_db.RegisterMessage(DeleteTrialsReply)

AddTrialRequest = _reflection.GeneratedProtocolMessageType('AddTrialRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDTRIALREQUEST,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.AddTrialRequest)
  })
_sym_db.RegisterMessage(AddTrialRequest)

AddTrialReply = _reflection.GeneratedProtocolMessageType('AddTrialReply', (_message.Message,), {
  'DESCRIPTOR' : _ADDTRIALREPLY,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.AddTrialReply)
  })
_sym_db.RegisterMessage(AddTrialReply)

AddSampleRequest = _reflection.GeneratedProtocolMessageType('AddSampleRequest', (_message.Message,), {
  'DESCRIPTOR' : _ADDSAMPLEREQUEST,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.AddSampleRequest)
  })
_sym_db.RegisterMessage(AddSampleRequest)

AddSamplesReply = _reflection.GeneratedProtocolMessageType('AddSamplesReply', (_message.Message,), {
  'DESCRIPTOR' : _ADDSAMPLESREPLY,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.AddSamplesReply)
  })
_sym_db.RegisterMessage(AddSamplesReply)

StoredTrialInfo = _reflection.GeneratedProtocolMessageType('StoredTrialInfo', (_message.Message,), {
  'DESCRIPTOR' : _STOREDTRIALINFO,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.StoredTrialInfo)
  })
_sym_db.RegisterMessage(StoredTrialInfo)

StoredTrialSample = _reflection.GeneratedProtocolMessageType('StoredTrialSample', (_message.Message,), {
  'DESCRIPTOR' : _STOREDTRIALSAMPLE,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.StoredTrialSample)
  })
_sym_db.RegisterMessage(StoredTrialSample)

StoredTrialActorSample = _reflection.GeneratedProtocolMessageType('StoredTrialActorSample', (_message.Message,), {
  'DESCRIPTOR' : _STOREDTRIALACTORSAMPLE,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.StoredTrialActorSample)
  })
_sym_db.RegisterMessage(StoredTrialActorSample)

StoredTrialActorSampleReward = _reflection.GeneratedProtocolMessageType('StoredTrialActorSampleReward', (_message.Message,), {
  'DESCRIPTOR' : _STOREDTRIALACTORSAMPLEREWARD,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.StoredTrialActorSampleReward)
  })
_sym_db.RegisterMessage(StoredTrialActorSampleReward)

StoredTrialActorSampleMessage = _reflection.GeneratedProtocolMessageType('StoredTrialActorSampleMessage', (_message.Message,), {
  'DESCRIPTOR' : _STOREDTRIALACTORSAMPLEMESSAGE,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.StoredTrialActorSampleMessage)
  })
_sym_db.RegisterMessage(StoredTrialActorSampleMessage)

TrialSamplesFileHeader = _reflection.GeneratedProtocolMessageType('TrialSamplesFileHeader', (_message.Message,), {

  'TrialParamsEntry' : _reflection.GeneratedProtocolMessageType('TrialParamsEntry', (_message.Message,), {
    'DESCRIPTOR' : _TRIALSAMPLESFILEHEADER_TRIALPARAMSENTRY,
    '__module__' : 'cogment.api.trial_datastore_pb2'
    # @@protoc_insertion_point(class_scope:cogmentAPI.TrialSamplesFileHeader.TrialParamsEntry)
    })
  ,
  'DESCRIPTOR' : _TRIALSAMPLESFILEHEADER,
  '__module__' : 'cogment.api.trial_datastore_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.TrialSamplesFileHeader)
  })
_sym_db.RegisterMessage(TrialSamplesFileHeader)
_sym_db.RegisterMessage(TrialSamplesFileHeader.TrialParamsEntry)

_TRIALDATASTORESP = DESCRIPTOR.services_by_name['TrialDatastoreSP']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z.github.com/cogment/cogment/grpcapi/cogment/api'
  _TRIALSAMPLESFILEHEADER_TRIALPARAMSENTRY._options = None
  _TRIALSAMPLESFILEHEADER_TRIALPARAMSENTRY._serialized_options = b'8\001'
  _STOREDTRIALSAMPLEFIELD._serialized_start=2066
  _STOREDTRIALSAMPLEFIELD._serialized_end=2434
  _RETRIEVETRIALSREQUEST._serialized_start=75
  _RETRIEVETRIALSREQUEST._serialized_end=178
  _RETRIEVETRIALSREPLY._serialized_start=180
  _RETRIEVETRIALSREPLY._serialized_end=278
  _RETRIEVESAMPLESREQUEST._serialized_start=281
  _RETRIEVESAMPLESREQUEST._serialized_end=467
  _RETRIEVESAMPLEREPLY._serialized_start=469
  _RETRIEVESAMPLEREPLY._serialized_end=543
  _DELETETRIALSREQUEST._serialized_start=545
  _DELETETRIALSREQUEST._serialized_end=585
  _DELETETRIALSREPLY._serialized_start=587
  _DELETETRIALSREPLY._serialized_end=606
  _ADDTRIALREQUEST._serialized_start=608
  _ADDTRIALREQUEST._serialized_end=689
  _ADDTRIALREPLY._serialized_start=691
  _ADDTRIALREPLY._serialized_end=706
  _ADDSAMPLEREQUEST._serialized_start=708
  _ADDSAMPLEREQUEST._serialized_end=779
  _ADDSAMPLESREPLY._serialized_start=781
  _ADDSAMPLESREPLY._serialized_end=798
  _STOREDTRIALINFO._serialized_start=801
  _STOREDTRIALINFO._serialized_end=961
  _STOREDTRIALSAMPLE._serialized_start=964
  _STOREDTRIALSAMPLE._serialized_end=1170
  _STOREDTRIALACTORSAMPLE._serialized_start=1173
  _STOREDTRIALACTORSAMPLE._serialized_end=1586
  _STOREDTRIALACTORSAMPLEREWARD._serialized_start=1589
  _STOREDTRIALACTORSAMPLEREWARD._serialized_end=1727
  _STOREDTRIALACTORSAMPLEMESSAGE._serialized_start=1729
  _STOREDTRIALACTORSAMPLEMESSAGE._serialized_end=1811
  _TRIALSAMPLESFILEHEADER._serialized_start=1814
  _TRIALSAMPLESFILEHEADER._serialized_end=2063
  _TRIALSAMPLESFILEHEADER_TRIALPARAMSENTRY._serialized_start=1988
  _TRIALSAMPLESFILEHEADER_TRIALPARAMSENTRY._serialized_end=2063
  _TRIALDATASTORESP._serialized_start=2437
  _TRIALDATASTORESP._serialized_end=2929
# @@protoc_insertion_point(module_scope)



# Copyright 2021 AI Redefined Inc. <dev+cogment@ai-r.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import cogment.api.environment_pb2 as env_api

from cogment.session import Session, _Ending, _EndingAck
from cogment.errors import CogmentError
from cogment.utils import logger

import time


class EnvironmentSession(Session):
    """Derived class representing the session of an environment for a trial."""

    def __init__(self, impl, trial, name, impl_name, config):
        super().__init__(trial, name, impl, impl_name, config)

    def __str__(self):
        result = super().__str__()
        result += f" --- EnvironmentSession: config = {self.config}"
        return result

    def get_active_actors(self):
        return self._active_actors

    def send_message(self, payload, to, to_environment=None):
        if to_environment is not None:
            logger.deprecated("Parameter 'to_environment' is deprecated for 'send_message' method. "
                              "No message will be sent back to the environment!")
        self._send_message(payload, to)

    def start(self, observations=None, auto_done_sending=True):
        self._start(auto_done_sending)

        if observations is not None:
            logger.debug(f"Trial [{self._trial.id}] - Environment [{self.name}] Sending first observation.")
            packed_obs = self._pack_observations(observations, -1)
            self._post_outgoing_data(packed_obs)

    def produce_observations(self, observations):
        if not self._trial.ended:
            packed_obs = self._pack_observations(observations, -1)
            self._post_outgoing_data(packed_obs)
            if self._trial.ending and self._auto_ack:
                self._post_outgoing_data(_EndingAck())
        else:
            logger.warning(f"Trial [{self._trial.id}] - Environment [{self.name}] "
                           f"Cannot send observation because the trial has ended.")

    def end(self, final_observations):
        if self._trial.ended:
            logger.warning(f"Trial [{self._trial.id}] - Environment [{self.name}] "
                           f"end request ignored because the trial has already ended.")
        elif self._trial.ending_ack:
            logger.warning(f"Trial [{self._trial.id}] - Environment [{self.name}] cannot end more than once")
        else:
            if not self._trial.ending:
                self._post_outgoing_data(_Ending())
            packed_obs = self._pack_observations(final_observations, -1)
            self._post_outgoing_data(packed_obs)
            if self._auto_ack:
                self._post_outgoing_data(_EndingAck())

    def _pack_observations(self, observations, tick_id):
        timestamp = int(time.time() * 1e9)

        new_obs = [None] * len(self._trial.actors)

        for target, obs in observations:
            if not isinstance(target, str):
                raise CogmentError(f"Target actor name [{target}] must be a string")

            if target == "*" or target == "*.*":
                if len(observations) > 1:
                    raise CogmentError(f"Duplicate actors in observations list when using a wildcard")
                new_obs = [obs] * len(self._trial.actors)
                break
            else:
                for actor_index, actor in enumerate(self._trial.actors):
                    if "." not in target:
                        if actor.name == target:
                            if new_obs[actor_index] is not None:
                                raise CogmentError(f"Duplicate actor [{actor.name}] in observations list")
                            new_obs[actor_index] = obs
                            break  # Names are unique in trial
                    else:
                        class_name, actor_name = target.split(".")
                        if class_name == actor.actor_class.name:
                            if actor_name == actor.name or actor_name == "*":
                                if new_obs[actor_index] is not None:
                                    raise CogmentError(f"Duplicate actor [{class_name}.{actor.name}] "
                                                       f"in observations list")
                                new_obs[actor_index] = obs

        for actor_index, actor in enumerate(self._trial.actors):
            if new_obs[actor_index] is None:
                raise CogmentError(f"Actor [{actor.name}] is missing an observation")

        package = env_api.ObservationSet()
        package.tick_id = tick_id
        package.timestamp = timestamp

        seen_observations = {}
        for actor_index, actor in enumerate(self._trial.actors):
            actor_obs = new_obs[actor_index]
            obs_id = id(actor_obs)
            obs_key = seen_observations.get(obs_id)
            if obs_key is None:
                obs_key = len(package.observations)

                package.observations.append(actor_obs.SerializeToString())

                seen_observations[obs_id] = obs_key

            package.actors_map.append(obs_key)

        return package



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cogment/api/model_registry.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cogment.api import common_pb2 as cogment_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n cogment/api/model_registry.proto\x12\ncogmentAPI\x1a\x18\x63ogment/api/common.proto\"\x86\x01\n\tModelInfo\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x36\n\tuser_data\x18\x02 \x03(\x0b\x32#.cogmentAPI.ModelInfo.UserDataEntry\x1a/\n\rUserDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x80\x02\n\x10ModelVersionInfo\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x16\n\x0eversion_number\x18\x02 \x01(\r\x12\x1a\n\x12\x63reation_timestamp\x18\x03 \x01(\x06\x12\x10\n\x08\x61rchived\x18\x04 \x01(\x08\x12\x11\n\tdata_hash\x18\x05 \x01(\t\x12\x11\n\tdata_size\x18\x06 \x01(\x06\x12=\n\tuser_data\x18\x07 \x03(\x0b\x32*.cogmentAPI.ModelVersionInfo.UserDataEntry\x1a/\n\rUserDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"G\n\x1a\x43reateOrUpdateModelRequest\x12)\n\nmodel_info\x18\x01 \x01(\x0b\x32\x15.cogmentAPI.ModelInfo\"\x1a\n\x18\x43reateOrUpdateModelReply\"V\n\x15RetrieveModelsRequest\x12\x11\n\tmodel_ids\x18\x01 \x03(\t\x12\x14\n\x0cmodels_count\x18\x03 \x01(\r\x12\x14\n\x0cmodel_handle\x18\x04 \x01(\t\"\\\n\x13RetrieveModelsReply\x12*\n\x0bmodel_infos\x18\x01 \x03(\x0b\x32\x15.cogmentAPI.ModelInfo\x12\x19\n\x11next_model_handle\x18\x02 \x01(\t\"&\n\x12\x44\x65leteModelRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\"\x12\n\x10\x44\x65leteModelReply\"\xf8\x01\n\x19\x43reateVersionRequestChunk\x12>\n\x06header\x18\x01 \x01(\x0b\x32,.cogmentAPI.CreateVersionRequestChunk.HeaderH\x00\x12:\n\x04\x62ody\x18\x02 \x01(\x0b\x32*.cogmentAPI.CreateVersionRequestChunk.BodyH\x00\x1a<\n\x06Header\x12\x32\n\x0cversion_info\x18\x01 \x01(\x0b\x32\x1c.cogmentAPI.ModelVersionInfo\x1a\x1a\n\x04\x42ody\x12\x12\n\ndata_chunk\x18\x01 \x01(\x0c\x42\x05\n\x03msg\"H\n\x12\x43reateVersionReply\x12\x32\n\x0cversion_info\x18\x01 \x01(\x0b\x32\x1c.cogmentAPI.ModelVersionInfo\"x\n\x1bRetrieveVersionInfosRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x17\n\x0fversion_numbers\x18\x02 \x03(\x05\x12\x16\n\x0eversions_count\x18\x03 \x01(\r\x12\x16\n\x0eversion_handle\x18\x04 \x01(\t\"m\n\x19RetrieveVersionInfosReply\x12\x33\n\rversion_infos\x18\x01 \x03(\x0b\x32\x1c.cogmentAPI.ModelVersionInfo\x12\x1b\n\x13next_version_handle\x18\x02 \x01(\t\"F\n\x1aRetrieveVersionDataRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x16\n\x0eversion_number\x18\x02 \x01(\x05\"3\n\x1dRetrieveVersionDataReplyChunk\x12\x12\n\ndata_chunk\x18\x01 \x01(\x0c\x32\x95\x05\n\x0fModelRegistrySP\x12\x65\n\x13\x43reateOrUpdateModel\x12&.cogmentAPI.CreateOrUpdateModelRequest\x1a$.cogmentAPI.CreateOrUpdateModelReply\"\x00\x12M\n\x0b\x44\x65leteModel\x12\x1e.cogmentAPI.DeleteModelRequest\x1a\x1c.cogmentAPI.DeleteModelReply\"\x00\x12V\n\x0eRetrieveModels\x12!.cogmentAPI.RetrieveModelsRequest\x1a\x1f.cogmentAPI.RetrieveModelsReply\"\x00\x12Z\n\rCreateVersion\x12%.cogmentAPI.CreateVersionRequestChunk\x1a\x1e.cogmentAPI.CreateVersionReply\"\x00(\x01\x12h\n\x14RetrieveVersionInfos\x12\'.cogmentAPI.RetrieveVersionInfosRequest\x1a%.cogmentAPI.RetrieveVersionInfosReply\"\x00\x12l\n\x13RetrieveVersionData\x12&.cogmentAPI.RetrieveVersionDataRequest\x1a).cogmentAPI.RetrieveVersionDataReplyChunk\"\x00\x30\x01\x12@\n\x07Version\x12\x1a.cogmentAPI.VersionRequest\x1a\x17.cogmentAPI.VersionInfo\"\x00\x42\x30Z.github.com/cogment/cogment/grpcapi/cogment/apib\x06proto3')



_MODELINFO = DESCRIPTOR.message_types_by_name['ModelInfo']
_MODELINFO_USERDATAENTRY = _MODELINFO.nested_types_by_name['UserDataEntry']
_MODELVERSIONINFO = DESCRIPTOR.message_types_by_name['ModelVersionInfo']
_MODELVERSIONINFO_USERDATAENTRY = _MODELVERSIONINFO.nested_types_by_name['UserDataEntry']
_CREATEORUPDATEMODELREQUEST = DESCRIPTOR.message_types_by_name['CreateOrUpdateModelRequest']
_CREATEORUPDATEMODELREPLY = DESCRIPTOR.message_types_by_name['CreateOrUpdateModelReply']
_RETRIEVEMODELSREQUEST = DESCRIPTOR.message_types_by_name['RetrieveModelsRequest']
_RETRIEVEMODELSREPLY = DESCRIPTOR.message_types_by_name['RetrieveModelsReply']
_DELETEMODELREQUEST = DESCRIPTOR.message_types_by_name['DeleteModelRequest']
_DELETEMODELREPLY = DESCRIPTOR.message_types_by_name['DeleteModelReply']
_CREATEVERSIONREQUESTCHUNK = DESCRIPTOR.message_types_by_name['CreateVersionRequestChunk']
_CREATEVERSIONREQUESTCHUNK_HEADER = _CREATEVERSIONREQUESTCHUNK.nested_types_by_name['Header']
_CREATEVERSIONREQUESTCHUNK_BODY = _CREATEVERSIONREQUESTCHUNK.nested_types_by_name['Body']
_CREATEVERSIONREPLY = DESCRIPTOR.message_types_by_name['CreateVersionReply']
_RETRIEVEVERSIONINFOSREQUEST = DESCRIPTOR.message_types_by_name['RetrieveVersionInfosRequest']
_RETRIEVEVERSIONINFOSREPLY = DESCRIPTOR.message_types_by_name['RetrieveVersionInfosReply']
_RETRIEVEVERSIONDATAREQUEST = DESCRIPTOR.message_types_by_name['RetrieveVersionDataRequest']
_RETRIEVEVERSIONDATAREPLYCHUNK = DESCRIPTOR.message_types_by_name['RetrieveVersionDataReplyChunk']
ModelInfo = _reflection.GeneratedProtocolMessageType('ModelInfo', (_message.Message,), {

  'UserDataEntry' : _reflection.GeneratedProtocolMessageType('UserDataEntry', (_message.Message,), {
    'DESCRIPTOR' : _MODELINFO_USERDATAENTRY,
    '__module__' : 'cogment.api.model_registry_pb2'
    # @@protoc_insertion_point(class_scope:cogmentAPI.ModelInfo.UserDataEntry)
    })
  ,
  'DESCRIPTOR' : _MODELINFO,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ModelInfo)
  })
_sym_db.RegisterMessage(ModelInfo)
_sym_db.RegisterMessage(ModelInfo.UserDataEntry)

ModelVersionInfo = _reflection.GeneratedProtocolMessageType('ModelVersionInfo', (_message.Message,), {

  'UserDataEntry' : _reflection.GeneratedProtocolMessageType('UserDataEntry', (_message.Message,), {
    'DESCRIPTOR' : _MODELVERSIONINFO_USERDATAENTRY,
    '__module__' : 'cogment.api.model_registry_pb2'
    # @@protoc_insertion_point(class_scope:cogmentAPI.ModelVersionInfo.UserDataEntry)
    })
  ,
  'DESCRIPTOR' : _MODELVERSIONINFO,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.ModelVersionInfo)
  })
_sym_db.RegisterMessage(ModelVersionInfo)
_sym_db.RegisterMessage(ModelVersionInfo.UserDataEntry)

CreateOrUpdateModelRequest = _reflection.GeneratedProtocolMessageType('CreateOrUpdateModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEORUPDATEMODELREQUEST,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.CreateOrUpdateModelRequest)
  })
_sym_db.RegisterMessage(CreateOrUpdateModelRequest)

CreateOrUpdateModelReply = _reflection.GeneratedProtocolMessageType('CreateOrUpdateModelReply', (_message.Message,), {
  'DESCRIPTOR' : _CREATEORUPDATEMODELREPLY,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.CreateOrUpdateModelReply)
  })
_sym_db.RegisterMessage(CreateOrUpdateModelReply)

RetrieveModelsRequest = _reflection.GeneratedProtocolMessageType('RetrieveModelsRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEMODELSREQUEST,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveModelsRequest)
  })
_sym_db.RegisterMessage(RetrieveModelsRequest)

RetrieveModelsReply = _reflection.GeneratedProtocolMessageType('RetrieveModelsReply', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEMODELSREPLY,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveModelsReply)
  })
_sym_db.RegisterMessage(RetrieveModelsReply)

DeleteModelRequest = _reflection.GeneratedProtocolMessageType('DeleteModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEMODELREQUEST,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.DeleteModelRequest)
  })
_sym_db.RegisterMessage(DeleteModelRequest)

DeleteModelReply = _reflection.GeneratedProtocolMessageType('DeleteModelReply', (_message.Message,), {
  'DESCRIPTOR' : _DELETEMODELREPLY,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.DeleteModelReply)
  })
_sym_db.RegisterMessage(DeleteModelReply)

CreateVersionRequestChunk = _reflection.GeneratedProtocolMessageType('CreateVersionRequestChunk', (_message.Message,), {

  'Header' : _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
    'DESCRIPTOR' : _CREATEVERSIONREQUESTCHUNK_HEADER,
    '__module__' : 'cogment.api.model_registry_pb2'
    # @@protoc_insertion_point(class_scope:cogmentAPI.CreateVersionRequestChunk.Header)
    })
  ,

  'Body' : _reflection.GeneratedProtocolMessageType('Body', (_message.Message,), {
    'DESCRIPTOR' : _CREATEVERSIONREQUESTCHUNK_BODY,
    '__module__' : 'cogment.api.model_registry_pb2'
    # @@protoc_insertion_point(class_scope:cogmentAPI.CreateVersionRequestChunk.Body)
    })
  ,
  'DESCRIPTOR' : _CREATEVERSIONREQUESTCHUNK,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.CreateVersionRequestChunk)
  })
_sym_db.RegisterMessage(CreateVersionRequestChunk)
_sym_db.RegisterMessage(CreateVersionRequestChunk.Header)
_sym_db.RegisterMessage(CreateVersionRequestChunk.Body)

CreateVersionReply = _reflection.GeneratedProtocolMessageType('CreateVersionReply', (_message.Message,), {
  'DESCRIPTOR' : _CREATEVERSIONREPLY,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.CreateVersionReply)
  })
_sym_db.RegisterMessage(CreateVersionReply)

RetrieveVersionInfosRequest = _reflection.GeneratedProtocolMessageType('RetrieveVersionInfosRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEVERSIONINFOSREQUEST,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveVersionInfosRequest)
  })
_sym_db.RegisterMessage(RetrieveVersionInfosRequest)

RetrieveVersionInfosReply = _reflection.GeneratedProtocolMessageType('RetrieveVersionInfosReply', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEVERSIONINFOSREPLY,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveVersionInfosReply)
  })
_sym_db.RegisterMessage(RetrieveVersionInfosReply)

RetrieveVersionDataRequest = _reflection.GeneratedProtocolMessageType('RetrieveVersionDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEVERSIONDATAREQUEST,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveVersionDataRequest)
  })
_sym_db.RegisterMessage(RetrieveVersionDataRequest)

RetrieveVersionDataReplyChunk = _reflection.GeneratedProtocolMessageType('RetrieveVersionDataReplyChunk', (_message.Message,), {
  'DESCRIPTOR' : _RETRIEVEVERSIONDATAREPLYCHUNK,
  '__module__' : 'cogment.api.model_registry_pb2'
  # @@protoc_insertion_point(class_scope:cogmentAPI.RetrieveVersionDataReplyChunk)
  })
_sym_db.RegisterMessage(RetrieveVersionDataReplyChunk)

_MODELREGISTRYSP = DESCRIPTOR.services_by_name['ModelRegistrySP']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z.github.com/cogment/cogment/grpcapi/cogment/api'
  _MODELINFO_USERDATAENTRY._options = None
  _MODELINFO_USERDATAENTRY._serialized_options = b'8\001'
  _MODELVERSIONINFO_USERDATAENTRY._options = None
  _MODELVERSIONINFO_USERDATAENTRY._serialized_options = b'8\001'
  _MODELINFO._serialized_start=75
  _MODELINFO._serialized_end=209
  _MODELINFO_USERDATAENTRY._serialized_start=162
  _MODELINFO_USERDATAENTRY._serialized_end=209
  _MODELVERSIONINFO._serialized_start=212
  _MODELVERSIONINFO._serialized_end=468
  _MODELVERSIONINFO_USERDATAENTRY._serialized_start=162
  _MODELVERSIONINFO_USERDATAENTRY._serialized_end=209
  _CREATEORUPDATEMODELREQUEST._serialized_start=470
  _CREATEORUPDATEMODELREQUEST._serialized_end=541
  _CREATEORUPDATEMODELREPLY._serialized_start=543
  _CREATEORUPDATEMODELREPLY._serialized_end=569
  _RETRIEVEMODELSREQUEST._serialized_start=571
  _RETRIEVEMODELSREQUEST._serialized_end=657
  _RETRIEVEMODELSREPLY._serialized_start=659
  _RETRIEVEMODELSREPLY._serialized_end=751
  _DELETEMODELREQUEST._serialized_start=753
  _DELETEMODELREQUEST._serialized_end=791
  _DELETEMODELREPLY._serialized_start=793
  _DELETEMODELREPLY._serialized_end=811
  _CREATEVERSIONREQUESTCHUNK._serialized_start=814
  _CREATEVERSIONREQUESTCHUNK._serialized_end=1062
  _CREATEVERSIONREQUESTCHUNK_HEADER._serialized_start=967
  _CREATEVERSIONREQUESTCHUNK_HEADER._serialized_end=1027
  _CREATEVERSIONREQUESTCHUNK_BODY._serialized_start=1029
  _CREATEVERSIONREQUESTCHUNK_BODY._serialized_end=1055
  _CREATEVERSIONREPLY._serialized_start=1064
  _CREATEVERSIONREPLY._serialized_end=1136
  _RETRIEVEVERSIONINFOSREQUEST._serialized_start=1138
  _RETRIEVEVERSIONINFOSREQUEST._serialized_end=1258
  _RETRIEVEVERSIONINFOSREPLY._serialized_start=1260
  _RETRIEVEVERSIONINFOSREPLY._serialized_end=1369
  _RETRIEVEVERSIONDATAREQUEST._serialized_start=1371
  _RETRIEVEVERSIONDATAREQUEST._serialized_end=1441
  _RETRIEVEVERSIONDATAREPLYCHUNK._serialized_start=1443
  _RETRIEVEVERSIONDATAREPLYCHUNK._serialized_end=1494
  _MODELREGISTRYSP._serialized_start=1497
  _MODELREGISTRYSP._serialized_end=2158
# @@protoc_insertion_point(module_scope)
